#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Integrity flag messages
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize
from sbp.gnss import GPSTimeSec, SvId

# Automatically generated from piksi/yaml/swiftnav/sbp/integrity.yaml with generate.py.
# Please do not hand edit!


class IntegritySSRHeader(object):
  """IntegritySSRHeader.
  
  
  Parameters
  ----------
  obs_time : GPSTimeSec
    GNSS reference time of the observation used to generate the flag.
  num_msgs : int
    Number of messages in the dataset
  seq_num : int
    Position of this message in the dataset
  ssr_sol_id : int
    SSR Solution ID.
  tile_set_id : int
    Unique identifier of the set this tile belongs to.
  tile_id : int
    Unique identifier of this tile in the tile set.
  chain_id : int
    Chain and type of flag.

  """
  _parser = construct.Struct(
                     'obs_time' / GPSTimeSec._parser,
                     'num_msgs' / construct.Int8ul,
                     'seq_num' / construct.Int8ul,
                     'ssr_sol_id' / construct.Int8ul,
                     'tile_set_id' / construct.Int16ul,
                     'tile_id' / construct.Int16ul,
                     'chain_id' / construct.Int8ul,)
  __slots__ = [
               'obs_time',
               'num_msgs',
               'seq_num',
               'ssr_sol_id',
               'tile_set_id',
               'tile_id',
               'chain_id',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.obs_time = kwargs.pop('obs_time')
      self.num_msgs = kwargs.pop('num_msgs')
      self.seq_num = kwargs.pop('seq_num')
      self.ssr_sol_id = kwargs.pop('ssr_sol_id')
      self.tile_set_id = kwargs.pop('tile_set_id')
      self.tile_id = kwargs.pop('tile_id')
      self.chain_id = kwargs.pop('chain_id')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = IntegritySSRHeader._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
SBP_MSG_SSR_FLAG_HIGH_LEVEL = 0x0BB9
class MsgSsrFlagHighLevel(SBP):
  """SBP class for message MSG_SSR_FLAG_HIGH_LEVEL (0x0BB9).

  You can have MSG_SSR_FLAG_HIGH_LEVEL inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Integrity monitoring flags for multiple aggregated elements. An element
  could be a satellite, SSR grid point, or SSR tile. A group of aggregated
  elements being monitored for integrity could refer to:

  - Satellites in a particular {GPS, GAL, BDS} constellation.

  - Satellites in the line-of-sight of a particular SSR tile.

  - Satellites in the line-of-sight of a particular SSR grid point.

  The integrity usage for a group of aggregated elements varies according to
  the integrity flag of the satellites comprising that group.

  SSR_INTEGRITY_USAGE_NOMINAL: All satellites received passed the integrity
  check and have flag INTEGRITY_FLAG_OK.

  SSR_INTEGRITY_USAGE_WARNING: A limited number of elements in the group
  failed the integrity check. Refer to more granular integrity messages for
  details on the specific failing elements.

  SSR_INTEGRITY_USAGE_ALERT: Most elements in the group failed the integrity
  check, do not use for positioning.

  SSR_INTEGRITY_USAGE_NOT_MONITORED: Unable to verify the integrity flag of
  elements in the group.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  obs_time : GPSTimeSec
    GNSS reference time of the observation used to generate the flag.
  corr_time : GPSTimeSec
    GNSS reference time of the correction associated to the flag.
  ssr_sol_id : int
    SSR Solution ID.
  tile_set_id : int
    Unique identifier of the set this tile belongs to.
  tile_id : int
    Unique identifier of this tile in the tile set.
  chain_id : int
    Chain and type of flag.
  use_gps_sat : int
    Use GPS satellites.
  use_gal_sat : int
    Use GAL satellites.
  use_bds_sat : int
    Use BDS satellites.
  reserved : array
    Reserved
  use_tropo_grid_points : int
    Use tropo grid points.
  use_iono_grid_points : int
    Use iono grid points.
  use_iono_tile_sat_los : int
    Use iono tile satellite LoS.
  use_iono_grid_point_sat_los : int
    Use iono grid point satellite LoS.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'obs_time' / GPSTimeSec._parser,
                   'corr_time' / GPSTimeSec._parser,
                   'ssr_sol_id' / construct.Int8ul,
                   'tile_set_id' / construct.Int16ul,
                   'tile_id' / construct.Int16ul,
                   'chain_id' / construct.Int8ul,
                   'use_gps_sat' / construct.Int8ul,
                   'use_gal_sat' / construct.Int8ul,
                   'use_bds_sat' / construct.Int8ul,
                   'reserved' / construct.Array(6, construct.Int8ul),
                   'use_tropo_grid_points' / construct.Int8ul,
                   'use_iono_grid_points' / construct.Int8ul,
                   'use_iono_tile_sat_los' / construct.Int8ul,
                   'use_iono_grid_point_sat_los' / construct.Int8ul,)
  __slots__ = [
               'obs_time',
               'corr_time',
               'ssr_sol_id',
               'tile_set_id',
               'tile_id',
               'chain_id',
               'use_gps_sat',
               'use_gal_sat',
               'use_bds_sat',
               'reserved',
               'use_tropo_grid_points',
               'use_iono_grid_points',
               'use_iono_tile_sat_los',
               'use_iono_grid_point_sat_los',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagHighLevel,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagHighLevel, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_HIGH_LEVEL
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.obs_time = kwargs.pop('obs_time')
      self.corr_time = kwargs.pop('corr_time')
      self.ssr_sol_id = kwargs.pop('ssr_sol_id')
      self.tile_set_id = kwargs.pop('tile_set_id')
      self.tile_id = kwargs.pop('tile_id')
      self.chain_id = kwargs.pop('chain_id')
      self.use_gps_sat = kwargs.pop('use_gps_sat')
      self.use_gal_sat = kwargs.pop('use_gal_sat')
      self.use_bds_sat = kwargs.pop('use_bds_sat')
      self.reserved = kwargs.pop('reserved')
      self.use_tropo_grid_points = kwargs.pop('use_tropo_grid_points')
      self.use_iono_grid_points = kwargs.pop('use_iono_grid_points')
      self.use_iono_tile_sat_los = kwargs.pop('use_iono_tile_sat_los')
      self.use_iono_grid_point_sat_los = kwargs.pop('use_iono_grid_point_sat_los')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagHighLevel.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagHighLevel(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagHighLevel._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagHighLevel._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagHighLevel._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagHighLevel, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SSR_FLAG_SATELLITES = 0x0BBD
class MsgSsrFlagSatellites(SBP):
  """SBP class for message MSG_SSR_FLAG_SATELLITES (0x0BBD).

  You can have MSG_SSR_FLAG_SATELLITES inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  obs_time : GPSTimeSec
    GNSS reference time of the observation used to generate the flag.
  num_msgs : int
    Number of messages in the dataset
  seq_num : int
    Position of this message in the dataset
  ssr_sol_id : int
    SSR Solution ID.
  chain_id : int
    Chain and type of flag.
  const_id : int
    Constellation ID.
  n_faulty_sats : int
    Number of faulty satellites.
  faulty_sats : array
    List of faulty satellites.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'obs_time' / GPSTimeSec._parser,
                   'num_msgs' / construct.Int8ul,
                   'seq_num' / construct.Int8ul,
                   'ssr_sol_id' / construct.Int8ul,
                   'chain_id' / construct.Int8ul,
                   'const_id' / construct.Int8ul,
                   'n_faulty_sats' / construct.Int8ul,
                   'faulty_sats' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'obs_time',
               'num_msgs',
               'seq_num',
               'ssr_sol_id',
               'chain_id',
               'const_id',
               'n_faulty_sats',
               'faulty_sats',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagSatellites,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagSatellites, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_SATELLITES
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.obs_time = kwargs.pop('obs_time')
      self.num_msgs = kwargs.pop('num_msgs')
      self.seq_num = kwargs.pop('seq_num')
      self.ssr_sol_id = kwargs.pop('ssr_sol_id')
      self.chain_id = kwargs.pop('chain_id')
      self.const_id = kwargs.pop('const_id')
      self.n_faulty_sats = kwargs.pop('n_faulty_sats')
      self.faulty_sats = kwargs.pop('faulty_sats')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagSatellites.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagSatellites(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagSatellites._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagSatellites._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagSatellites._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagSatellites, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SSR_FLAG_TROPO_GRID_POINTS = 0x0BC3
class MsgSsrFlagTropoGridPoints(SBP):
  """SBP class for message MSG_SSR_FLAG_TROPO_GRID_POINTS (0x0BC3).

  You can have MSG_SSR_FLAG_TROPO_GRID_POINTS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  header : IntegritySSRHeader
    Header of an integrity message.
  n_faulty_points : int
    Number of faulty grid points.
  faulty_points : array
    List of faulty grid points.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'header' / IntegritySSRHeader._parser,
                   'n_faulty_points' / construct.Int8ul,
                   'faulty_points' / construct.GreedyRange(construct.Int16ul),)
  __slots__ = [
               'header',
               'n_faulty_points',
               'faulty_points',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagTropoGridPoints,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagTropoGridPoints, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_TROPO_GRID_POINTS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.header = kwargs.pop('header')
      self.n_faulty_points = kwargs.pop('n_faulty_points')
      self.faulty_points = kwargs.pop('faulty_points')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagTropoGridPoints.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagTropoGridPoints(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagTropoGridPoints._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagTropoGridPoints._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagTropoGridPoints._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagTropoGridPoints, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SSR_FLAG_IONO_GRID_POINTS = 0x0BC7
class MsgSsrFlagIonoGridPoints(SBP):
  """SBP class for message MSG_SSR_FLAG_IONO_GRID_POINTS (0x0BC7).

  You can have MSG_SSR_FLAG_IONO_GRID_POINTS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  header : IntegritySSRHeader
    Header of an integrity message.
  n_faulty_points : int
    Number of faulty grid points.
  faulty_points : array
    List of faulty grid points.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'header' / IntegritySSRHeader._parser,
                   'n_faulty_points' / construct.Int8ul,
                   'faulty_points' / construct.GreedyRange(construct.Int16ul),)
  __slots__ = [
               'header',
               'n_faulty_points',
               'faulty_points',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagIonoGridPoints,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagIonoGridPoints, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_IONO_GRID_POINTS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.header = kwargs.pop('header')
      self.n_faulty_points = kwargs.pop('n_faulty_points')
      self.faulty_points = kwargs.pop('faulty_points')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagIonoGridPoints.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagIonoGridPoints(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagIonoGridPoints._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagIonoGridPoints._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagIonoGridPoints._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagIonoGridPoints, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SSR_FLAG_IONO_TILE_SAT_LOS = 0x0BCD
class MsgSsrFlagIonoTileSatLos(SBP):
  """SBP class for message MSG_SSR_FLAG_IONO_TILE_SAT_LOS (0x0BCD).

  You can have MSG_SSR_FLAG_IONO_TILE_SAT_LOS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  header : IntegritySSRHeader
    Header of an integrity message.
  n_faulty_los : int
    Number of faulty LOS.
  faulty_los : array
    List of faulty LOS
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'header' / IntegritySSRHeader._parser,
                   'n_faulty_los' / construct.Int8ul,
                   'faulty_los' / construct.GreedyRange(SvId._parser),)
  __slots__ = [
               'header',
               'n_faulty_los',
               'faulty_los',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagIonoTileSatLos,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagIonoTileSatLos, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_IONO_TILE_SAT_LOS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.header = kwargs.pop('header')
      self.n_faulty_los = kwargs.pop('n_faulty_los')
      self.faulty_los = kwargs.pop('faulty_los')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagIonoTileSatLos.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagIonoTileSatLos(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagIonoTileSatLos._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagIonoTileSatLos._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagIonoTileSatLos._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagIonoTileSatLos, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SSR_FLAG_IONO_GRID_POINT_SAT_LOS = 0x0BD1
class MsgSsrFlagIonoGridPointSatLos(SBP):
  """SBP class for message MSG_SSR_FLAG_IONO_GRID_POINT_SAT_LOS (0x0BD1).

  You can have MSG_SSR_FLAG_IONO_GRID_POINT_SAT_LOS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  header : IntegritySSRHeader
    Header of an integrity message.
  grid_point_id : int
    Index of the grid point.
  n_faulty_los : int
    Number of faulty LOS.
  faulty_los : array
    List of faulty LOS
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'header' / IntegritySSRHeader._parser,
                   'grid_point_id' / construct.Int16ul,
                   'n_faulty_los' / construct.Int8ul,
                   'faulty_los' / construct.GreedyRange(SvId._parser),)
  __slots__ = [
               'header',
               'grid_point_id',
               'n_faulty_los',
               'faulty_los',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSsrFlagIonoGridPointSatLos,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSsrFlagIonoGridPointSatLos, self).__init__()
      self.msg_type = SBP_MSG_SSR_FLAG_IONO_GRID_POINT_SAT_LOS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.header = kwargs.pop('header')
      self.grid_point_id = kwargs.pop('grid_point_id')
      self.n_faulty_los = kwargs.pop('n_faulty_los')
      self.faulty_los = kwargs.pop('faulty_los')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSsrFlagIonoGridPointSatLos.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSsrFlagIonoGridPointSatLos(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSsrFlagIonoGridPointSatLos._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSsrFlagIonoGridPointSatLos._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSsrFlagIonoGridPointSatLos._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSsrFlagIonoGridPointSatLos, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0BB9: MsgSsrFlagHighLevel,
  0x0BBD: MsgSsrFlagSatellites,
  0x0BC3: MsgSsrFlagTropoGridPoints,
  0x0BC7: MsgSsrFlagIonoGridPoints,
  0x0BCD: MsgSsrFlagIonoTileSatLos,
  0x0BD1: MsgSsrFlagIonoGridPointSatLos,
}


#!/usr/bin/env python

#
# Python setuptools specification for Swift Navigation's libsbp
#
# Version handling code pulled from Enthought's TraitsUI:
#   https://github.com/swift-nav/traitsui/blob/swift-2019.01/setup.py
#

import warnings
from setuptools import setup, find_packages

import re
import os

import subprocess

CLASSIFIERS = [
  'Intended Audience :: Developers',
  'Intended Audience :: Science/Research',
  'Operating System :: POSIX :: Linux',
  'Operating System :: MacOS :: MacOS X',
  'Operating System :: Microsoft :: Windows',
  'Programming Language :: Python',
  'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
  'Topic :: Software Development :: Libraries :: Python Modules',
  'Programming Language :: Python :: 3.6',
  'Programming Language :: Python :: 3.7',
  'Programming Language :: Python :: 3.8',
  'Programming Language :: Python :: 3.9',
  'Programming Language :: Python :: 3.10',
]

PACKAGES = find_packages(exclude=["tests", "bench", "tests.*"])

PLATFORMS = [
  'linux',
  'osx',
  'win32',
]

setup_py_dir = os.path.dirname(os.path.abspath(__file__))

def _read_release_version():
    relver_path = os.path.join(setup_py_dir, 'sbp/RELEASE-VERSION')
    try:
        with open(relver_path, "r") as f:
            version = f.readlines()[0]
            return version.strip()
    except IOError as ex:
        warnings.warn("Error reading version: {}".format(ex))
        return "0.0.0"


def read_release_version():
    major, minor, patch = _read_release_version().split('.')
    return (int(major), int(minor), patch)


MAJOR, MINOR, PATCH = read_release_version()
IS_RELEASED = os.environ.get('IS_RELEASED', '') != ''

VERSION = '%d.%d.%s' % (MAJOR, MINOR, PATCH)

VERSION_PY_TEMPLATE = """\
# THIS FILE IS GENERATED FROM TRAITS SETUP.PY
version = '{version}'
full_version = '{full_version}'
git_revision = '{git_revision}'
is_released = {is_released}
if not is_released:
    version = full_version
"""

VERSION_PY_PATH = 'sbp/_version.py'


# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd, stdout=subprocess.PIPE, env=env,
        ).communicate()[0]
        return out
    try:
        out = _minimal_ext_cmd(['git', 'describe', '--tags'])
    except OSError:
        out = b''
    git_description = out.strip().decode('ascii')
    expr = r'.*?\-(?P<count>\d+)-g(?P<hash>[a-fA-F0-9]+)'
    match = re.match(expr, git_description)
    if match is None:
        git_revision, git_count = 'Unknown', '0'
    else:
        git_revision, git_count = match.group('hash'), match.group('count')
    return git_revision, git_count


def write_version_py(filename=VERSION_PY_PATH):

    fullversion = VERSION
    if os.path.exists(os.path.join(setup_py_dir, '..', '.git')):
        git_rev, dev_num = git_version()
    elif os.path.exists(VERSION_PY_PATH):
        # must be a source distribution, use existing version file
        try:
            from sbp._version import git_revision as git_rev
            from sbp._version import full_version as full_v
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "sbp/_version.py and the build directory "
                              "before building.")

        match = re.match(r'.*?\.dev(?P<dev_num>\d+)', full_v)
        if match is None:
            dev_num = '0'
        else:
            dev_num = match.group('dev_num')
    else:
        git_rev = 'Unknown'
        dev_num = '0'

    if not IS_RELEASED:
        # According to the original source for this setup.py
        # version handling (see link at top of the file):
        #
        # We may need to add the git rev number outside of
        # write_version_py(), otherwise the import of ._version
        # might mess up the build under Python 3.
        #
        fullversion += '.dev{0}+g{1}'.format(dev_num, git_rev)

    filename_fullpath = os.path.join(setup_py_dir, filename)
    print(filename_fullpath)

    with open(filename_fullpath, "wt") as fp:
        fp.write(VERSION_PY_TEMPLATE.format(version=VERSION,
                                            full_version=fullversion,
                                            git_revision=git_rev,
                                            is_released=IS_RELEASED))


if __name__ == "__main__":

    with open(os.path.join(setup_py_dir, 'README.rst'), 'rb') as f:
        readme = f.read().decode('utf8')

    write_version_py()

    from sbp import __version__ as sbp_version
    print("Building/installing libsbp version {0} (read version: {1})".format(sbp_version, VERSION))

    ext_modules = None

    with open(os.path.join(setup_py_dir, 'requirements.txt')) as f:
        INSTALL_REQUIRES = [i.strip() for i in f.readlines()]

    with open(os.path.join(setup_py_dir, 'test_requirements.txt')) as f:
        TEST_REQUIRES = [i.strip() for i in f.readlines()]

    print("Discovered packages: {0}".format(PACKAGES))

    setup(
        name='sbp',
        version=sbp_version,
        description='Python bindings for Swift Binary Protocol',
        long_description=readme,
        author='Swift Navigation',
        author_email='dev@swift-nav.com',
        url='https://github.com/swift-nav/libsbp',
        classifiers=CLASSIFIERS,
        packages=PACKAGES,
        platforms=PLATFORMS,
        install_requires=INSTALL_REQUIRES,
        tests_require=TEST_REQUIRES,
        use_2to3=False,
        zip_safe=False,
        ext_modules=ext_modules,
        scripts=['bin/sbp2json'],
        extras_require={
          'sbp2json': [
              'python-rapidjson~=1.0',
              'pybase64~=1.1',
          ]
        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Author: Douglas Creager <dcreager@dcreager.net>
# This file is placed into the public domain.
# Adapted from Gist: https://gist.github.com/3067886

# Calculates the current version number.  If possible, this is the
# output of “git describe”, modified to conform to the versioning
# scheme that setuptools uses.  If “git describe” returns an error
# (most likely because we're in an unpacked copy of a release tarball,
# rather than in a git working copy), then we fall back on reading the
# contents of the RELEASE-VERSION file.
#
# To use this script, simply import it your setup.py file, and use the
# results of get_git_version() as your package version:
#
# from version import *
#
# setup(
#     version=get_git_version(),
#     .
#     .
#     .
# )
#
# This will automatically update the RELEASE-VERSION file, if
# necessary.  Note that the RELEASE-VERSION file should *not* be
# checked into git; please add it to your top-level .gitignore file.
#
# You'll probably want to distribute the RELEASE-VERSION file in your
# sdist tarballs; to do this, just create a MANIFEST.in file that
# contains the following line:
#
#   include RELEASE-VERSION

__all__ = ("get_git_version", )

from subprocess import Popen, PIPE


def call_git_describe():
    try:
        p = Popen(['git', 'describe', '--tags', '--dirty', '--always'],
                  stdout=PIPE, stderr=PIPE)
        p.stderr.close()
        line = p.stdout.readlines()[0].decode('ascii')
        return line.strip()
    except Exception:
        return None


def read_release_version():
    try:
        from sbp import __version__ as version
        return version
    except Exception:
        return None


def get_git_version():
    # Read in the version that's currently in RELEASE-VERSION.
    release_version = read_release_version()

    # First try to get the current version using “git describe”.
    version = call_git_describe()

    # Take off the leading if present.
    if version is not None and version[0] == 'v':
        version = version[1:]

    # Adapt to PEP 386 compatible versioning scheme
    version = pep386adapt(version)

    # If that doesn't work, fall back on the value that's in
    # RELEASE-VERSION.
    if version is None:
        version = release_version

    # If we still don't have anything, that's an error.
    if version is None:
        raise ValueError("Cannot find the version number!")

    # Finally, return the current version.
    return version


def pep386adapt(version):
    if version is not None and '-' in version:
        # adapt git-describe version to be in line with PEP 386
        # Break PEP 386 a bit here and append the Git hash
        parts = version.split('-')
        if len(parts) > 2:
            version = '%s.dev%s+%s' % (parts[0], parts[1], '-'.join(parts[2:]))
        return version
    else:
        return version


VERSION = get_git_version()


if __name__ == "__main__":
    print(VERSION)



#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Messages from a vehicle.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/vehicle.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_ODOMETRY = 0x0903
class MsgOdometry(SBP):
  """SBP class for message MSG_ODOMETRY (0x0903).

  You can have MSG_ODOMETRY inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Message representing the x component of vehicle velocity in the user frame
  at the odometry reference point(s) specified by the user. The offset for the
  odometry reference point and the definition and origin of the user frame are
  defined through the device settings interface. There are 4 possible user-
  defined sources of this message which are labeled arbitrarily source 0
  through 3. If using "processor time" time tags, the receiving end will
  expect a `MSG_GNSS_TIME_OFFSET` when a PVT fix becomes available to
  synchronise odometry measurements with GNSS. Processor time shall roll over
  to zero after one week.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  tow : int
    Time field representing either milliseconds in the GPS Week or local CPU
    time from the producing system in milliseconds.  See the tow_source flag
    for the exact source of this timestamp.
  velocity : int
    The signed forward component of vehicle velocity.
  flags : int
    Status flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'tow' / construct.Int32ul,
                   'velocity' / construct.Int32sl,
                   'flags' / construct.Int8ul,)
  __slots__ = [
               'tow',
               'velocity',
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgOdometry,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgOdometry, self).__init__()
      self.msg_type = SBP_MSG_ODOMETRY
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.tow = kwargs.pop('tow')
      self.velocity = kwargs.pop('velocity')
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgOdometry.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgOdometry(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgOdometry._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgOdometry._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgOdometry._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgOdometry, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_WHEELTICK = 0x0904
class MsgWheeltick(SBP):
  """SBP class for message MSG_WHEELTICK (0x0904).

  You can have MSG_WHEELTICK inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Message containing the accumulated distance travelled by a wheel located at
  an odometry reference point defined by the user. The offset for the odometry
  reference point and the definition and origin of the user frame are defined
  through the device settings interface. The source of this message is
  identified by the source field, which is an integer ranging from 0 to 255.
  The timestamp associated with this message should represent the time when
  the accumulated tick count reached the value given by the contents of this
  message as accurately as possible. If using "local CPU time" time tags, the
  receiving end will expect a `MSG_GNSS_TIME_OFFSET` when a PVT fix becomes
  available to synchronise wheeltick measurements with GNSS. Local CPU time
  shall roll over to zero after one week.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  time : int
    Time field representing either microseconds since the last PPS,
    microseconds in the GPS Week or local CPU time from the producing system
    in microseconds. See the synch_type field for the exact meaning of this
    timestamp.
  flags : int
    Field indicating the type of timestamp contained in the time field.
  source : int
    ID of the sensor producing this message
  ticks : int
    Free-running counter of the accumulated distance for this sensor. The
    counter should be incrementing if travelling into one direction and
    decrementing when travelling in the opposite direction.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'time' / construct.Int64ul,
                   'flags' / construct.Int8ul,
                   'source' / construct.Int8ul,
                   'ticks' / construct.Int32sl,)
  __slots__ = [
               'time',
               'flags',
               'source',
               'ticks',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgWheeltick,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgWheeltick, self).__init__()
      self.msg_type = SBP_MSG_WHEELTICK
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.time = kwargs.pop('time')
      self.flags = kwargs.pop('flags')
      self.source = kwargs.pop('source')
      self.ticks = kwargs.pop('ticks')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgWheeltick.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgWheeltick(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgWheeltick._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgWheeltick._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgWheeltick._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgWheeltick, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0903: MsgOdometry,
  0x0904: MsgWheeltick,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Standardized system messages from Swift Navigation devices.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/system.yaml with generate.py.
# Please do not hand edit!


class SubSystemReport(object):
  """SubSystemReport.
  
  Report the general and specific state of a subsystem.  If the generic state
  is reported as initializing, the specific state should be ignored.
  
  Parameters
  ----------
  component : int
    Identity of reporting subsystem
  generic : int
    Generic form status report
  specific : int
    Subsystem specific status code

  """
  _parser = construct.Struct(
                     'component' / construct.Int16ul,
                     'generic' / construct.Int8ul,
                     'specific' / construct.Int8ul,)
  __slots__ = [
               'component',
               'generic',
               'specific',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.component = kwargs.pop('component')
      self.generic = kwargs.pop('generic')
      self.specific = kwargs.pop('specific')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = SubSystemReport._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class StatusJournalItem(object):
  """StatusJournalItem.
  
  Reports the uptime and the state of a subsystem via generic and specific
  status codes.  If the generic state is reported as initializing, the
  specific state should be ignored.
  
  Parameters
  ----------
  uptime : int
    Milliseconds since system startup
  report : SubSystemReport

  """
  _parser = construct.Struct(
                     'uptime' / construct.Int32ul,
                     'report' / SubSystemReport._parser,)
  __slots__ = [
               'uptime',
               'report',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.uptime = kwargs.pop('uptime')
      self.report = kwargs.pop('report')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = StatusJournalItem._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
SBP_MSG_STARTUP = 0xFF00
class MsgStartup(SBP):
  """SBP class for message MSG_STARTUP (0xFF00).

  You can have MSG_STARTUP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The system start-up message is sent once on system start-up. It notifies the
  host or other attached devices that the system has started and is now ready
  to respond to commands or configuration requests.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  cause : int
    Cause of startup
  startup_type : int
    Startup type
  reserved : int
    Reserved
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'cause' / construct.Int8ul,
                   'startup_type' / construct.Int8ul,
                   'reserved' / construct.Int16ul,)
  __slots__ = [
               'cause',
               'startup_type',
               'reserved',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStartup,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStartup, self).__init__()
      self.msg_type = SBP_MSG_STARTUP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.cause = kwargs.pop('cause')
      self.startup_type = kwargs.pop('startup_type')
      self.reserved = kwargs.pop('reserved')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStartup.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStartup(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStartup._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStartup._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStartup._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStartup, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_DGNSS_STATUS = 0xFF02
class MsgDgnssStatus(SBP):
  """SBP class for message MSG_DGNSS_STATUS (0xFF02).

  You can have MSG_DGNSS_STATUS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message provides information about the receipt of Differential
  corrections.  It is expected to be sent with each receipt of a complete
  corrections packet.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Status flags
  latency : int
    Latency of observation receipt
  num_signals : int
    Number of signals from base station
  source : string
    Corrections source string
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int8ul,
                   'latency' / construct.Int16ul,
                   'num_signals' / construct.Int8ul,
                   'source' / construct.GreedyBytes,)
  __slots__ = [
               'flags',
               'latency',
               'num_signals',
               'source',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgDgnssStatus,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgDgnssStatus, self).__init__()
      self.msg_type = SBP_MSG_DGNSS_STATUS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')
      self.latency = kwargs.pop('latency')
      self.num_signals = kwargs.pop('num_signals')
      self.source = kwargs.pop('source')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgDgnssStatus.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgDgnssStatus(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgDgnssStatus._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgDgnssStatus._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgDgnssStatus._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgDgnssStatus, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_HEARTBEAT = 0xFFFF
class MsgHeartbeat(SBP):
  """SBP class for message MSG_HEARTBEAT (0xFFFF).

  You can have MSG_HEARTBEAT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The heartbeat message is sent periodically to inform the host or other
  attached devices that the system is running. It is used to monitor system
  malfunctions. It also contains status flags that indicate to the host the
  status of the system and whether it is operating correctly. Currently, the
  expected heartbeat interval is 1 sec.

  The system error flag is used to indicate that an error has occurred in the
  system. To determine the source of the error, the remaining error flags
  should be inspected.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Status flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int32ul,)
  __slots__ = [
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgHeartbeat,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgHeartbeat, self).__init__()
      self.msg_type = SBP_MSG_HEARTBEAT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgHeartbeat.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgHeartbeat(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgHeartbeat._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgHeartbeat._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgHeartbeat._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgHeartbeat, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_STATUS_REPORT = 0xFFFE
class MsgStatusReport(SBP):
  """SBP class for message MSG_STATUS_REPORT (0xFFFE).

  You can have MSG_STATUS_REPORT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The status report is sent periodically to inform the host or other attached
  devices that the system is running. It is used to monitor system
  malfunctions. It contains status reports that indicate to the host the
  status of each subsystem and whether it is operating correctly.

  Interpretation of the subsystem specific status code is product dependent,
  but if the generic status code is initializing, it should be ignored.  Refer
  to product documentation for details.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  reporting_system : int
    Identity of reporting system
  sbp_version : int
    SBP protocol version
  sequence : int
    Increments on each status report sent
  uptime : int
    Number of seconds since system start-up
  status : array
    Reported status of individual subsystems
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'reporting_system' / construct.Int16ul,
                   'sbp_version' / construct.Int16ul,
                   'sequence' / construct.Int32ul,
                   'uptime' / construct.Int32ul,
                   'status' / construct.GreedyRange(SubSystemReport._parser),)
  __slots__ = [
               'reporting_system',
               'sbp_version',
               'sequence',
               'uptime',
               'status',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStatusReport,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStatusReport, self).__init__()
      self.msg_type = SBP_MSG_STATUS_REPORT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.reporting_system = kwargs.pop('reporting_system')
      self.sbp_version = kwargs.pop('sbp_version')
      self.sequence = kwargs.pop('sequence')
      self.uptime = kwargs.pop('uptime')
      self.status = kwargs.pop('status')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStatusReport.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStatusReport(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStatusReport._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStatusReport._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStatusReport._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStatusReport, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_STATUS_JOURNAL = 0xFFFD
class MsgStatusJournal(SBP):
  """SBP class for message MSG_STATUS_JOURNAL (0xFFFD).

  You can have MSG_STATUS_JOURNAL inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The status journal message contains past status reports (see
  MSG_STATUS_REPORT) and functions as a error/event storage for telemetry
  purposes.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  reporting_system : int
    Identity of reporting system
  sbp_version : int
    SBP protocol version
  total_status_reports : int
    Total number of status reports sent since system startup
  sequence_descriptor : int
    Index and number of messages in this sequence. First nibble is the size of
    the sequence (n), second nibble is the zero-indexed counter (ith packet of
    n)
  journal : array
    Status journal
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'reporting_system' / construct.Int16ul,
                   'sbp_version' / construct.Int16ul,
                   'total_status_reports' / construct.Int32ul,
                   'sequence_descriptor' / construct.Int8ul,
                   'journal' / construct.GreedyRange(StatusJournalItem._parser),)
  __slots__ = [
               'reporting_system',
               'sbp_version',
               'total_status_reports',
               'sequence_descriptor',
               'journal',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStatusJournal,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStatusJournal, self).__init__()
      self.msg_type = SBP_MSG_STATUS_JOURNAL
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.reporting_system = kwargs.pop('reporting_system')
      self.sbp_version = kwargs.pop('sbp_version')
      self.total_status_reports = kwargs.pop('total_status_reports')
      self.sequence_descriptor = kwargs.pop('sequence_descriptor')
      self.journal = kwargs.pop('journal')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStatusJournal.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStatusJournal(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStatusJournal._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStatusJournal._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStatusJournal._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStatusJournal, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_INS_STATUS = 0xFF03
class MsgInsStatus(SBP):
  """SBP class for message MSG_INS_STATUS (0xFF03).

  You can have MSG_INS_STATUS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The INS status message describes the state of the operation and
  initialization of the inertial navigation system.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Status flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int32ul,)
  __slots__ = [
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgInsStatus,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgInsStatus, self).__init__()
      self.msg_type = SBP_MSG_INS_STATUS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgInsStatus.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgInsStatus(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgInsStatus._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgInsStatus._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgInsStatus._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgInsStatus, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_CSAC_TELEMETRY = 0xFF04
class MsgCsacTelemetry(SBP):
  """SBP class for message MSG_CSAC_TELEMETRY (0xFF04).

  You can have MSG_CSAC_TELEMETRY inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The CSAC telemetry message has an implementation defined telemetry string
  from a device. It is not produced or available on general Swift Products. It
  is intended to be a low rate message for status purposes.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  id : int
    Index representing the type of telemetry in use.  It is implementation
    defined.
  telemetry : string
    Comma separated list of values as defined by the index
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'id' / construct.Int8ul,
                   'telemetry' / construct.GreedyBytes,)
  __slots__ = [
               'id',
               'telemetry',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgCsacTelemetry,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgCsacTelemetry, self).__init__()
      self.msg_type = SBP_MSG_CSAC_TELEMETRY
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.id = kwargs.pop('id')
      self.telemetry = kwargs.pop('telemetry')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgCsacTelemetry.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgCsacTelemetry(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgCsacTelemetry._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgCsacTelemetry._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgCsacTelemetry._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgCsacTelemetry, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_CSAC_TELEMETRY_LABELS = 0xFF05
class MsgCsacTelemetryLabels(SBP):
  """SBP class for message MSG_CSAC_TELEMETRY_LABELS (0xFF05).

  You can have MSG_CSAC_TELEMETRY_LABELS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The CSAC telemetry message provides labels for each member of the string
  produced by MSG_CSAC_TELEMETRY. It should be provided by a device at a lower
  rate than the MSG_CSAC_TELEMETRY.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  id : int
    Index representing the type of telemetry in use.  It is implementation
    defined.
  telemetry_labels : string
    Comma separated list of telemetry field values
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'id' / construct.Int8ul,
                   'telemetry_labels' / construct.GreedyBytes,)
  __slots__ = [
               'id',
               'telemetry_labels',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgCsacTelemetryLabels,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgCsacTelemetryLabels, self).__init__()
      self.msg_type = SBP_MSG_CSAC_TELEMETRY_LABELS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.id = kwargs.pop('id')
      self.telemetry_labels = kwargs.pop('telemetry_labels')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgCsacTelemetryLabels.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgCsacTelemetryLabels(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgCsacTelemetryLabels._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgCsacTelemetryLabels._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgCsacTelemetryLabels._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgCsacTelemetryLabels, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_INS_UPDATES = 0xFF06
class MsgInsUpdates(SBP):
  """SBP class for message MSG_INS_UPDATES (0xFF06).

  You can have MSG_INS_UPDATES inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The INS update status message contains information about executed and
  rejected INS updates. This message is expected to be extended in the future
  as new types of measurements are being added.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  tow : int
    GPS Time of Week
  gnsspos : int
    GNSS position update status flags
  gnssvel : int
    GNSS velocity update status flags
  wheelticks : int
    Wheelticks update status flags
  speed : int
    Wheelticks update status flags
  nhc : int
    NHC update status flags
  zerovel : int
    Zero velocity update status flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'tow' / construct.Int32ul,
                   'gnsspos' / construct.Int8ul,
                   'gnssvel' / construct.Int8ul,
                   'wheelticks' / construct.Int8ul,
                   'speed' / construct.Int8ul,
                   'nhc' / construct.Int8ul,
                   'zerovel' / construct.Int8ul,)
  __slots__ = [
               'tow',
               'gnsspos',
               'gnssvel',
               'wheelticks',
               'speed',
               'nhc',
               'zerovel',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgInsUpdates,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgInsUpdates, self).__init__()
      self.msg_type = SBP_MSG_INS_UPDATES
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.tow = kwargs.pop('tow')
      self.gnsspos = kwargs.pop('gnsspos')
      self.gnssvel = kwargs.pop('gnssvel')
      self.wheelticks = kwargs.pop('wheelticks')
      self.speed = kwargs.pop('speed')
      self.nhc = kwargs.pop('nhc')
      self.zerovel = kwargs.pop('zerovel')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgInsUpdates.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgInsUpdates(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgInsUpdates._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgInsUpdates._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgInsUpdates._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgInsUpdates, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_GNSS_TIME_OFFSET = 0xFF07
class MsgGnssTimeOffset(SBP):
  """SBP class for message MSG_GNSS_TIME_OFFSET (0xFF07).

  You can have MSG_GNSS_TIME_OFFSET inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The GNSS time offset message contains the information that is needed to
  translate messages tagged with a local timestamp (e.g. IMU or wheeltick
  messages) to GNSS time for the sender producing this message.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  weeks : int
    Weeks portion of the time offset
  milliseconds : int
    Milliseconds portion of the time offset
  microseconds : int
    Microseconds portion of the time offset
  flags : int
    Status flags (reserved)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'weeks' / construct.Int16sl,
                   'milliseconds' / construct.Int32sl,
                   'microseconds' / construct.Int16sl,
                   'flags' / construct.Int8ul,)
  __slots__ = [
               'weeks',
               'milliseconds',
               'microseconds',
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgGnssTimeOffset,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgGnssTimeOffset, self).__init__()
      self.msg_type = SBP_MSG_GNSS_TIME_OFFSET
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.weeks = kwargs.pop('weeks')
      self.milliseconds = kwargs.pop('milliseconds')
      self.microseconds = kwargs.pop('microseconds')
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgGnssTimeOffset.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgGnssTimeOffset(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgGnssTimeOffset._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgGnssTimeOffset._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgGnssTimeOffset._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgGnssTimeOffset, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_PPS_TIME = 0xFF08
class MsgPpsTime(SBP):
  """SBP class for message MSG_PPS_TIME (0xFF08).

  You can have MSG_PPS_TIME inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The PPS time message contains the value of the sender's local time in
  microseconds at the moment a pulse is detected on the PPS input. This is to
  be used for syncronisation of sensor data sampled with a local timestamp
  (e.g. IMU or wheeltick messages) where GNSS time is unknown to the sender.

  The local time used to timestamp the PPS pulse must be generated by the same
  clock which is used to timestamp the IMU/wheel sensor data and should follow
  the same roll-over rules.  A separate MSG_PPS_TIME message should be sent
  for each source of sensor data which uses PPS-relative timestamping.  The
  sender ID for each of these MSG_PPS_TIME messages should match the sender ID
  of the respective sensor data.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  time : int
    Local time in microseconds
  flags : int
    Status flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'time' / construct.Int64ul,
                   'flags' / construct.Int8ul,)
  __slots__ = [
               'time',
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgPpsTime,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgPpsTime, self).__init__()
      self.msg_type = SBP_MSG_PPS_TIME
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.time = kwargs.pop('time')
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgPpsTime.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgPpsTime(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgPpsTime._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgPpsTime._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgPpsTime._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgPpsTime, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_SENSOR_AID_EVENT = 0xFF09
class MsgSensorAidEvent(SBP):
  """SBP class for message MSG_SENSOR_AID_EVENT (0xFF09).

  You can have MSG_SENSOR_AID_EVENT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This diagnostic message contains state and update status information for all
  sensors that are being used by the fusion engine. This message will be
  generated asynchronously to the solution messages and will be emitted
  anytime a sensor update is being processed.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  time : int
    Update timestamp in milliseconds.
  sensor_type : int
    Sensor type
  sensor_id : int
    Sensor identifier
  sensor_state : int
    Reserved for future use
  n_available_meas : int
    Number of available measurements in this epoch
  n_attempted_meas : int
    Number of attempted measurements in this epoch
  n_accepted_meas : int
    Number of accepted measurements in this epoch
  flags : int
    Reserved for future use
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'time' / construct.Int32ul,
                   'sensor_type' / construct.Int8ul,
                   'sensor_id' / construct.Int16ul,
                   'sensor_state' / construct.Int8ul,
                   'n_available_meas' / construct.Int8ul,
                   'n_attempted_meas' / construct.Int8ul,
                   'n_accepted_meas' / construct.Int8ul,
                   'flags' / construct.Int32ul,)
  __slots__ = [
               'time',
               'sensor_type',
               'sensor_id',
               'sensor_state',
               'n_available_meas',
               'n_attempted_meas',
               'n_accepted_meas',
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSensorAidEvent,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSensorAidEvent, self).__init__()
      self.msg_type = SBP_MSG_SENSOR_AID_EVENT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.time = kwargs.pop('time')
      self.sensor_type = kwargs.pop('sensor_type')
      self.sensor_id = kwargs.pop('sensor_id')
      self.sensor_state = kwargs.pop('sensor_state')
      self.n_available_meas = kwargs.pop('n_available_meas')
      self.n_attempted_meas = kwargs.pop('n_attempted_meas')
      self.n_accepted_meas = kwargs.pop('n_accepted_meas')
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSensorAidEvent.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSensorAidEvent(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSensorAidEvent._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSensorAidEvent._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSensorAidEvent._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSensorAidEvent, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_GROUP_META = 0xFF0A
class MsgGroupMeta(SBP):
  """SBP class for message MSG_GROUP_META (0xFF0A).

  You can have MSG_GROUP_META inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This leading message lists the time metadata of the Solution Group. It also
  lists the atomic contents (i.e. types of messages included) of the Solution
  Group.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  group_id : int
    Id of the Msgs Group, 0 is Unknown, 1 is Bestpos, 2 is Gnss
  flags : int
    Status flags (reserved)
  n_group_msgs : int
    Size of list group_msgs
  group_msgs : array
    An in-order list of message types included in the Solution Group,
    including GROUP_META itself
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'group_id' / construct.Int8ul,
                   'flags' / construct.Int8ul,
                   'n_group_msgs' / construct.Int8ul,
                   'group_msgs' / construct.GreedyRange(construct.Int16ul),)
  __slots__ = [
               'group_id',
               'flags',
               'n_group_msgs',
               'group_msgs',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgGroupMeta,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgGroupMeta, self).__init__()
      self.msg_type = SBP_MSG_GROUP_META
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.group_id = kwargs.pop('group_id')
      self.flags = kwargs.pop('flags')
      self.n_group_msgs = kwargs.pop('n_group_msgs')
      self.group_msgs = kwargs.pop('group_msgs')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgGroupMeta.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgGroupMeta(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgGroupMeta._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgGroupMeta._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgGroupMeta._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgGroupMeta, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0xFF00: MsgStartup,
  0xFF02: MsgDgnssStatus,
  0xFFFF: MsgHeartbeat,
  0xFFFE: MsgStatusReport,
  0xFFFD: MsgStatusJournal,
  0xFF03: MsgInsStatus,
  0xFF04: MsgCsacTelemetry,
  0xFF05: MsgCsacTelemetryLabels,
  0xFF06: MsgInsUpdates,
  0xFF07: MsgGnssTimeOffset,
  0xFF08: MsgPpsTime,
  0xFF09: MsgSensorAidEvent,
  0xFF0A: MsgGroupMeta,
}


# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from .base_driver import BaseDriver
import serial
import serial.tools.list_ports
try:
    import termios
    SerialError = termios.error
except ImportError:
    SerialError = None

try:
    SerialTimeoutException = serial.SerialTimeoutException
except AttributeError:
    SerialTimeoutException = serial.writeTimeoutError  # pylint: disable=no-member


class PySerialDriver(BaseDriver):
    """
    PySerialDriver

    The :class:`PySerialDriver` class reads SBP messages from a serial port
    using the pyserial driver.  This is mostly redundant, is the Serial object's
    read and write methods can be used directly.

    Parameters
    ----------
    port : string
      URI to port to read SBP messages from. Accepts the following types
      of URLs:
      - rfc2217://<host>:<port>[/<option>[/<option>]]
      - socket://<host>:<port>[/<option>[/<option>]]
      - loop://[<option>[/<option>]]
      and device names, such as /dev/ttyUSB0 (Linux) and COM3 (Windows). See
      http://pyserial.sourceforge.net/pyserial_api.html#urls for more details.
    baud : int
      Baud rate of serial port (defaults to 115200)

    """

    def __init__(self, port, baud=115200, rtscts=False):
        import serial
        try:
            handle = serial.serial_for_url(port)
            handle.baudrate = baud
            handle.timeout = None
            handle.rtscts = rtscts
            super(PySerialDriver, self).__init__(handle)
        except (OSError, serial.SerialException) as e:
            print()
            print("Error opening serial device '%s':" % port)
            print(e)
            print()
            print("The following serial devices were detected:")
            print()
            for (name, desc, _) in serial.tools.list_ports.comports():
                if desc[0:4] == "ttyS":
                    continue
                if name == desc:
                    print("\t%s" % name)
                else:
                    print("\t%s (%s)" % (name, desc))
            print()
            raise SystemExit

    def _read(self, size):
        """
        Read wrapper.

        Parameters
        ----------
        size : int
          Number of bytes to read.
        """
        try:
            return self.handle.read(size)
        except (OSError, serial.SerialException):
            print()
            print("Piksi disconnected")
            print()
            self.close()
            raise IOError

    def _write(self, s):
        """
        Write wrapper.

        Parameters
        ----------
        s : bytes
          Bytes to write
        """
        try:
            return self.handle.write(s)
        except (OSError, serial.SerialException, SerialTimeoutException) as e:
            if e == SerialTimeoutException:
                print("sbp pyserial_driver: writeTimeoutError")
                return 0
            else:
                print()
                print("Piksi disconnected")
                print()
                self.close()
                raise IOError

    def __enter__(self):
        self.flush()
        return self

    def __exit__(self, *args):
        try:
            self.flush()
            self.close()
        except (OSError, SerialError, serial.SerialException):
            pass
